@@ -12,10 +12,10 @@
 #    See the License for the specific language governing permissions and
 #    limitations under the License.
 
-import json
-
 import six
 
+from oslo_serialization import jsonutils
+
 from mistralclient.api import base
 from mistralclient import utils
 
@@ -27,7 +27,7 @@ class Environment(base.Resource):
         """Override loading of the "variables" attribute from text to dict."""
         for k, v in self._data.items():
             if k == 'variables' and isinstance(v, six.string_types):
-                v = json.loads(v)
+                v = jsonutils.loads(v)
 
             try:
                 setattr(self, k, v)
@@ -51,7 +51,7 @@ class EnvironmentManager(base.ResourceManager):
 
         # Convert dict to text for the variables attribute.
         if isinstance(kwargs['variables'], dict):
-            kwargs['variables'] = json.dumps(kwargs['variables'])
+            kwargs['variables'] = jsonutils.dumps(kwargs['variables'])
 
         return self._create('/environments', kwargs)
 
@@ -67,12 +67,23 @@ class EnvironmentManager(base.ResourceManager):
 
         # Convert dict to text for the variables attribute.
         if kwargs.get('variables') and isinstance(kwargs['variables'], dict):
-            kwargs['variables'] = json.dumps(kwargs['variables'])
+            kwargs['variables'] = jsonutils.dumps(kwargs['variables'])
 
         return self._update('/environments', kwargs)
 
-    def list(self):
-        return self._list('/environments', response_key='environments')
+    def list(self, marker='', limit=None, sort_keys='', sort_dirs='',
+             fields='', **filters):
+        query_string = self._build_query_params(
+            marker=marker,
+            limit=limit,
+            sort_keys=sort_keys,
+            sort_dirs=sort_dirs,
+            fields=fields,
+            filters=filters
+        )
+
+        return self._list('/environments%s' % query_string,
+                          response_key='environments')
 
     def get(self, name):
         self._ensure_not_empty(name=name)
